@@ -1,8 +1,9 @@
-use crate::{request::Request, Result};
+use crate::{request::Request, strip_ansi_escapes, Result};
+use autolink;
 use htmlescape;
 use phetch::{
     gopher,
-    menu::{Line, Menu},
+    menu::{self, Line},
 };
 use rust_embed::RustEmbed;
 use std::{
@@ -125,38 +126,50 @@ fn render(req: &Request, content: &str) -> Result<String> {
 /// Fetch the Gopher response for a given URL or search term.
 /// A "search term" is anything that isn't a URL.
 fn fetch(url_or_search_term: &str) -> io::Result<String> {
-    gopher::fetch_url(&user_input_to_url(url_or_search_term))
+    gopher::fetch_url(&user_input_to_url(url_or_search_term), true, false)
+        .and_then(|res| Ok(filter_gopher_response(&res.1).to_string()))
+}
+
+/// Filter ANSI escape codes and other weirdness from Gopher
+/// responses, since we're displaying this in a web browser.
+/// Maybe in the future we could
+fn filter_gopher_response(input: &str) -> Cow<str> {
+    if let Ok(stripped) = strip_ansi_escapes::strip(input) {
+        if stripped.len() != input.len() {
+            if let Ok(stripped) = std::str::from_utf8(&stripped) {
+                return Cow::from(stripped.to_string());
+            }
+        }
+    }
+    Cow::from(input)
 }
 
 /// Parses user input from the search/url box into a Gopher URL. The
 /// input can either be a literal URL or a search term that is
 /// translated into a Veronica query.
-fn user_input_to_url(input: &str) -> Cow<str> {
+fn user_input_to_url(input: &str) -> String {
     // space and no slash means search query
     if input.contains(' ') && !input.contains('/') {
         search_url(input)
     } else if !input.contains('.') && !input.contains('/') {
         // no dot and no slash is also a search query
         search_url(input)
-    } else if input.starts_with("gopher://") {
-        // strip gopher:// from input
-        Cow::from(&input["gopher://".len()..])
     } else {
         // anything else is a url
-        Cow::from(input)
+        input.replace("%20", " ").replace("gopher://", "")
     }
 }
 
 /// Given a search term, returns a Gopher URL to search for it.
-fn search_url(query: &str) -> Cow<str> {
+fn search_url(query: &str) -> String {
     let mut out = "gopher.floodgap.com/7/v2/vs?".to_string();
     out.push_str(query);
-    Cow::from(out)
+    out
 }
 
 /// Convert a Gopher response into HTML (links, etc).
 fn to_html(url: &str, gopher: &str) -> String {
-    if let (gopher::Type::Text, _, _, _) = gopher::parse_url(url) {
+    if gopher::type_for_url(url).is_text() {
         to_text_html(url, gopher)
     } else {
         to_menu_html(url, gopher)
@@ -167,22 +180,22 @@ fn to_html(url: &str, gopher: &str) -> String {
 /// inline search fields.
 fn to_menu_html(url: &str, gopher: &str) -> String {
     let mut out = String::new();
-    let menu = Menu::parse(url.to_string(), gopher.to_string());
+    let menu = menu::parse(url, gopher.to_string());
     for line in menu.lines {
         out.push_str(&format!("<div class='line {:?}'>", line.typ));
-        if line.typ == gopher::Type::HTML {
-            out.push_str(format!("<a href='{}'>", line.url).as_ref());
-        } else if line.typ != gopher::Type::Info && line.typ != gopher::Type::Search {
-            out.push_str(format!("<a href='/{}'>", line.url).as_ref());
+        if line.typ.is_html() {
+            out.push_str(format!("<a href='{}'>", line.url(gopher)).as_ref());
+        } else if !line.typ.is_info() && line.typ != gopher::Type::Search {
+            out.push_str(format!("<a href='/{}'>", line.url(gopher)).as_ref());
         }
-        if line.name.is_empty() {
+        if line.text(gopher).is_empty() {
             out.push_str("&nbsp;");
         } else if line.typ == gopher::Type::Search {
-            out.push_str(&to_search_html(&line));
+            out.push_str(&to_search_html(&line, gopher));
         } else {
-            out.push_str(&htmlescape::encode_minimal(&line.name));
+            out.push_str(&htmlescape::encode_minimal(&line.text(gopher)));
         }
-        if line.typ != gopher::Type::Info && line.typ != gopher::Type::Search {
+        if !line.typ.is_info() && line.typ != gopher::Type::Search {
             out.push_str("</a>");
         }
         out.push_str("</div>");
@@ -202,20 +215,15 @@ fn to_text_html(_url: &str, gopher: &str) -> String {
 
 /// Autolink mailto, HTTP/S, and Gopher URLs in plain text.
 fn link_urls(input: &str) -> String {
-    let finder = linkify::LinkFinder::new();
-    let mut out = input.to_string();
-    for link in finder.links(&input) {
-        let url = link.as_str();
-        out = out.replace(url, &format!("<a href=\"{}\">{}</a>", url, url));
-    }
-    out.replace("href=\"gopher://", "href=\"/")
+    autolink::auto_link(input, &[]).replace("href=\"gopher://", "href=\"/")
 }
 
 /// HTML for a Gopher Search item.
-fn to_search_html(line: &Line) -> String {
+fn to_search_html(line: &Line, raw: &str) -> String {
     format!(
         "<form class='search' action='{}'><input width='100%' type='text' placeholder='{}'></form>",
-        line.url, line.name
+        line.url(raw),
+        line.text(raw)
     )
 }
 
@@ -254,7 +262,7 @@ mod tests {
             "gopher.floodgap.com/7/v2/vs?can dogs talk"
         );
         assert_eq!(
-            user_input_to_url("gopher.floodgap.com/7/v2/vs?can gophers smell"),
+            user_input_to_url("gopher.floodgap.com/7/v2/vs?can%20gophers%20smell"),
             "gopher.floodgap.com/7/v2/vs?can gophers smell"
         );
     }
@@ -275,5 +283,33 @@ mod tests {
             link_urls("Or this one: gopher://sdf.org/1/users/undo maybe"),
             "Or this one: <a href=\"/sdf.org/1/users/undo\">gopher://sdf.org/1/users/undo</a> maybe"
         );
+
+        assert_eq!(
+            link_urls("Check out https://this-link.com! And also https://this.one.io. Or this one: gopher://sdf.org/1/users/undo maybe"),
+            "Check out <a href=\"https://this-link.com\">https://this-link.com</a>! And also <a href=\"https://this.one.io\">https://this.one.io</a>. Or this one: <a href=\"/sdf.org/1/users/undo\">gopher://sdf.org/1/users/undo</a> maybe"
+        );
+    }
+
+    #[test]
+    fn test_filter_gopher_response() {
+        assert_eq!(filter_gopher_response("Testing 1 2 3"), "Testing 1 2 3");
+
+        assert_eq!(
+            filter_gopher_response("One\n\tTwo\r\nThree"),
+            "One\n\tTwo\r\nThree"
+        );
+
+        assert_eq!(
+            filter_gopher_response("Welcome to \x1b[93;1mphetch\x1b[0m!"),
+            "Welcome to phetch!"
+        );
+
+        assert_eq!(filter_gopher_response("
+i> likes	(null)	phkt.io	70
+i[0mkitty, fish, c, rust, gopher, arch	(null)	phkt.io	70
+i	(null)	phkt.io	70
+i> dislikes	(null)	phkt.io	70
+i[0mcilantro, fig newtons, big files	(null)	phkt.io	70
+i	(null)	phkt.io	70"), "\ni> likes\t(null)\tphkt.io\t70\nikitty, fish, c, rust, gopher, arch\t(null)\tphkt.io\t70\ni\t(null)\tphkt.io\t70\ni> dislikes\t(null)\tphkt.io\t70\nicilantro, fig newtons, big files\t(null)\tphkt.io\t70\ni\t(null)\tphkt.io\t70");
     }
 }
